var request = require('request')
var Ajv = require('ajv')
var ajv = new Ajv({allErrors: true})

var Communication = function() {}

var schema = {
    type: 'object',
    required: ['adminclass', 'shopID', 'domain', 'url'],
    properties: {
        adminclass: {type: 'string'},
        shopID: {type: 'string'},
        domain: {type: 'string'},
        url: {type: 'string'},
        body: {type: 'object'}
    }
}

Communication.prototype.sendGETRequest = function(key, object) {
    return new Promise(function(resolve, reject) {
        var valid = ajv.validate(schema, object)
        /*IFLOG*/if(process.env.AJV_ERROR_LOG == 'true') console.developer.log(ajv.errors)
        if(valid) {
            let URL = `${object.domain}/Request/outer/${object.url}`
            request.get({
                url: URL,
                headers: {
                    authorization: `Bearer ${object.adminclass}`,
                    'x-shopid': object.shopID || 'null'
                }
            }, function(err, res, body) {
                if(err) reject({func: '@sendAPIRequest', lCode: '-E1', type:'request-error', msg: 'Hiba történt a lekérés futása közben!'})
                else {
                    try {
                        body = JSON.parse(body)
                        body.key = key
                        resolve(body)
                    } catch(e) {
                        reject({func: '@sendAPIRequest', lCode: '-E2', type:'parse-error', msg: 'Hiba történt a válasz feldolgozása közben!'})
                    }
                }
            })
        }
        else {
            reject({func: '@sendAPIRequest', lCode: '-E3', type:'data-error', msg: 'Hibás adatokat adott meg!'})
        }
    })
}

Communication.prototype.sendPOSTRequest = function(key, object) {
    return new Promise(function(resolve, reject) {
        var valid = ajv.validate(schema, object)
        /*IFLOG*/if(process.env.AJV_ERROR_LOG == 'true') console.developer.log(ajv.errors)
        if(valid) {
            let URL = `${object.domain}/Request/outer/${object.url}`
            request.post({
                url: URL,
                headers: {
                    authorization: `Bearer ${object.adminclass}`,
                    'x-shopid': object.shopID || 'null'
                },
                form: object.body || {}
            }, function(err, res, body) {
                if(err) reject({func: '@sendAPIRequest', lCode: '-E1', type:'request-error', msg: 'Hiba történt a lekérés futása közben!'})
                else {
                    try {
                        body = JSON.parse(body)
                        body.key = key
                        resolve(body)
                    } catch(e) {
                        reject({func: '@sendAPIRequest', lCode: '-E2', type:'parse-error', msg: 'Hiba történt a válasz feldolgozása közben!'})
                    }
                }
            })
        }
        else {
            reject({func: '@sendPOSTRequest', lCode: '-E3', type:'data-error', msg: 'Hibás adatokat adott meg!'})
        }
    })
}

module.exports = Communication